from SILEXlight import silex_lib_gmsh as sx_gmsh

def test_WriteResult():
    # sx_gmsh.WriteResults(filename=,nodes=,elttype= ,fields=)
    assert True
    
def test_ReadGmshNodes():
    # sx_gmsh.ReadGmshNodes(file=,nbcoord=)
    assert True
    
def test_ReadGmshElements():
    # sx_gmsh.ReadGmshElements(file=,elmttype=,prop=)
    assert True